macro_rules! impl_index_display {
($ty:ty, $fmt:literal) => { ... };
}Expand description
Implements Display for a nonmax index type using a format string.
macro_rules! impl_index_display {
($ty:ty, $fmt:literal) => { ... };
}Implements Display for a nonmax index type using a format string.