Inst

Struct Inst 

pub(crate) struct Inst(NonMaxU32);
An EVM instruction index into [Bytecode] instructions.

Also known as ic, or instruction counter; not to be confused with SSA insts.

Tuple Fields

0: NonMaxU32

Implementations

Source

impl Inst

pub(crate) const MAX_INDEX: usize

The maximum representable index value.

pub(crate) const CHECKS_MAX_INDEX: bool = true

Whether this index type performs bounds checking.

pub(crate) const fn new(value: usize) -> Self

Create a new index from a usize value.

Panics

Panics if value > MAX_INDEX.

pub(crate) const fn from_raw(value: NonMaxU32) -> Self

Create an index from a raw NonMaxU32 value.

pub(crate) fn from_foreign<F: Idx>(value: F) -> Self

Convert an index from another index type.

pub(crate) const fn from_usize_unchecked(value: usize) -> Self

Create an index from a usize without bounds checking.

Safety

The caller must ensure value <= MAX_INDEX.

pub(crate) const fn from_raw_unchecked(raw: u32) -> Self

Create an index from a raw u32 without bounds checking.

Safety

The caller must ensure the value is not u32::MAX.

pub(crate) const fn from_usize(value: usize) -> Self

Create an index from a usize with bounds checking.

Panics

Panics if value > MAX_INDEX.

pub(crate) const fn index(self) -> usize

Get the index value as a usize.

pub(crate) const fn raw(self) -> NonMaxU32

Get the raw NonMaxU32 value.