InstData

Struct InstData 

pub(crate) struct InstData {
    pub(crate) opcode: u8,
    pub(crate) flags: InstFlags,
    base_gas: u16,
    stack_io: (u8, u8),
    pub(crate) data: u32,
    pub(crate) gas_section: GasSection,
    pub(crate) stack_section: StackSection,
}
A single instruction in the bytecode.

Fields

opcode: u8

The opcode byte.

flags: InstFlags

Flags.

base_gas: u16

The base gas cost of the opcode.

This may not be the final/full gas cost of the opcode as it may also have a dynamic cost.

stack_io: (u8, u8)

Stack inputs and outputs.

data: u32

Instruction-specific data, populated at parse time and refined by analysis. Interpretation depends on opcode; access via the typed methods on InstData:

  • PUSH0..=PUSH32, DUPN | SWAPN | EXCHANGE: encoded U256Imm.
  • PC: program counter of this instruction.
  • JUMPDEST: program counter in low 31 bits, reachable flag in high bit (InstData::JUMPDEST_REACHABLE).
  • JUMP | JUMPI with InstFlags::STATIC_JUMP: target instruction index.
  • otherwise: unused.
gas_section: GasSection

The gas section this instruction belongs to.

stack_section: StackSection

The stack section this instruction belongs to.

Implementations

Source

impl InstData

pub(crate) const JUMPDEST_REACHABLE: u32

High bit of data set when a JUMPDEST is a reachable static jump target.

fn new(opcode: u8) -> Self

Creates a new instruction data with the given opcode byte. Note that this may not be a valid instruction.

pub(crate) const fn imm_len(&self) -> u8

Returns the length of the immediate data of this instruction.

pub(crate) fn stack_io(&self) -> (u8, u8)

Returns the number of input and output stack elements of this instruction.

pub(crate) const fn to_op(&self) -> Opcode<'static>

Converts this instruction to a raw opcode. Note that the immediate data is not resolved.

pub(crate) fn is_jump(&self) -> bool

Returns true if this instruction is a jump instruction (JUMP/JUMPI).

pub(crate) fn is_static_jump(&self) -> bool

Returns true if this instruction is a jump instruction (JUMP/JUMPI), and the target known statically.

pub(crate) const fn is_jumpdest(&self) -> bool

Returns true if this instruction is a JUMPDEST.

pub(crate) fn is_reachable_jumpdest(&self, has_dynamic_jumps: bool) -> bool

Returns true if this instruction is a reachable JUMPDEST.

pub(crate) fn imm(&self) -> U256Imm

Returns the U256Imm of a PUSH*/DUPN/SWAPN/EXCHANGE instruction.

pub(crate) fn imm_byte(&self) -> u8

Returns the inline u8 immediate of a DUPN/SWAPN/EXCHANGE instruction.

pub(crate) fn pc_imm(&self) -> u32

Returns the program counter stored inline for a PC instruction.

pub(crate) fn jumpdest_pc(&self) -> u32

Returns the program counter of a JUMPDEST.

pub(crate) fn is_jumpdest_reachable(&self) -> bool

Returns true if this JUMPDEST is the target of a resolved static jump.

pub(crate) fn set_jumpdest_reachable(&mut self)

Marks this JUMPDEST as a reachable static jump target.

pub(crate) fn static_jump_target(&self) -> Inst

Returns the static target of a JUMP/JUMPI with InstFlags::STATIC_JUMP.

pub(crate) fn set_static_jump_target(&mut self, target: Inst)

Sets the static jump target. Caller must also set InstFlags::STATIC_JUMP.

pub(crate) fn is_stack_section_head(&self) -> bool

Returns true if this instruction starts a new stack section.

pub(crate) fn is_dead_code(&self) -> bool

Returns true if this instruction is dead code.

pub(crate) fn requires_gasleft(&self, spec_id: SpecId) -> bool

Returns true if this instruction requires to know gasleft(). Note that this does not include CALL and CREATE.

pub(crate) fn can_fall_through(&self) -> bool

Returns true if execution can fall through to the next sequential instruction.

pub(crate) fn is_branching(&self) -> bool

Returns true if we know that this instruction will branch or stop execution.

pub(crate) fn is_diverging(&self) -> bool

Returns true if we know that this instruction will stop execution.

pub(crate) const fn may_suspend(&self) -> bool

Returns true if this instruction may suspend execution.