pub(crate) struct InstFlags(<InstFlags as PublicFlags>::Internal);Expand description
[InstrData] flags.
Tuple Fields
0: <InstFlags as PublicFlags>::InternalImplementations
Sourceimpl InstFlags
impl InstFlags
Sourcepub const STATIC_JUMP: Self
pub const STATIC_JUMP: Self
The JUMP/JUMPI target is known at compile time.
Sourcepub const INVALID_JUMP: Self
pub const INVALID_JUMP: Self
The jump target is known to be invalid.
Always returns [InstructionResult::InvalidJump] at runtime.
Sourcepub const MULTI_JUMP: Self
pub const MULTI_JUMP: Self
The jump has multiple known targets (see Bytecode::multi_jump_targets).
The target value is still on the stack and must be popped and switched on at runtime.
Sourcepub const DISABLED: Self
pub const DISABLED: Self
The instruction is disabled in this EVM version.
Always returns [InstructionResult::NotActivated] at runtime.
Sourcepub const UNKNOWN: Self
pub const UNKNOWN: Self
The instruction is unknown.
Always returns [InstructionResult::NotFound] at runtime.
Sourcepub const NOOP: Self
pub const NOOP: Self
Instruction is a no-op: skip generating logic, but keep the gas calculation.
Sourcepub const STACK_SECTION_HEAD: Self
pub const STACK_SECTION_HEAD: Self
This instruction starts a new stack section.
Sourceimpl InstFlags
impl InstFlags
Sourcepub const fn bits(&self) -> u8
pub const fn bits(&self) -> u8
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u8) -> Option<Self>
pub const fn from_bits(bits: u8) -> Option<Self>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u8) -> Self
pub const fn from_bits_truncate(bits: u8) -> Self
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u8) -> Self
pub const fn from_bits_retain(bits: u8) -> Self
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<Self>
pub fn from_name(name: &str) -> Option<Self>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: Self) -> bool
pub const fn intersects(&self, other: Self) -> bool
Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: Self) -> bool
pub const fn contains(&self, other: Self) -> bool
Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn remove(&mut self, other: Self)
pub fn remove(&mut self, other: Self)
The intersection of a source flags value with the complement of a target flags
value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: Self)
pub fn toggle(&mut self, other: Self)
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: Self, value: bool)
pub fn set(&mut self, other: Self, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: Self) -> Self
pub const fn intersection(self, other: Self) -> Self
The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: Self) -> Self
pub const fn union(self, other: Self) -> Self
The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: Self) -> Self
pub const fn difference(self, other: Self) -> Self
The intersection of a source flags value with the complement of a target flags
value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: Self) -> Self
pub const fn symmetric_difference(self, other: Self) -> Self
The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> Self
pub const fn complement(self) -> Self
The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl InstFlags
impl InstFlags
Sourcepub const fn iter(&self) -> Iter<InstFlags>
pub const fn iter(&self) -> Iter<InstFlags>
Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<InstFlags>
pub const fn iter_names(&self) -> IterNames<InstFlags>
Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.